package server

import (
	"asapo_broker/database"
	"asapo_common/utils"
	"net/http"
	"net/url"
)

type GetStreamsParams = database.GetStreamsParams

func getFilterKey(vals url.Values) (string, bool) {
	filter := vals.Get("filter")
	if filter == "" {
		return database.Stream_filter_all, true
	}
	if !utils.StringInSlice(filter, []string{database.Stream_filter_all,
		database.Stream_filter_finished, database.Stream_filter_unfinished}) {
		return database.Stream_filter_all, false
	} else {
		return filter, true
	}
}

func routeGetStreams(w http.ResponseWriter, r *http.Request) {
	keys := r.URL.Query()

	detailed, ok := getBoolKey(keys, "detailed", true)
	if !ok {
		http.Error(w, "can not interpret value: detailed", http.StatusBadRequest)
		return
	}
	last, ok := getBoolKey(keys, "last", false)
	if !ok {
		w.WriteHeader(http.StatusBadRequest)
		return
	}
	single, ok := getBoolKey(keys, "single", false)
	if !ok {
		w.WriteHeader(http.StatusBadRequest)
		return
	}
	filter, ok := getFilterKey(keys)
	if !ok {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	extraParams := GetStreamsParams{From: keys.Get("from"),
		Filter:   filter,
		Detailed: detailed,
		Last:     last,
		Single:   single,
	}
	processRequest(w, r, "streams", extraParams, false)
}
