package server

import (
	"asapo_broker/database"
	"encoding/json"
	"io/ioutil"
	"net/http"
	"strconv"
)

type ExtraParamMessageOp struct {
	Id     int
	Op     string
	Params struct {
		DelayMs int
	} `json:",omitempty"`
}

func routeMessageOp(w http.ResponseWriter, r *http.Request) {
	body, err := ioutil.ReadAll(r.Body)
	if err != nil {
		http.Error(w, err.Error(), 500)
		return
	}

	id_str, ok := extractRequestParametersID(r)
	if !ok {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	id, err := strconv.Atoi(id_str)
	if err != nil {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	var op ExtraParamMessageOp
	err = json.Unmarshal(body, &op)
	if err != nil {
		http.Error(w, "failed to deserialize request body: "+err.Error(), http.StatusBadRequest)
		return
	}

	var extraParam interface{}
	if op.Op == "ackmessage" {
		extraParam = database.ExtraParamAck{id}
	} else if op.Op == "negackmessage" {
		extraParam = database.ExtraParamNegAck{id, op.Params.DelayMs}
	} else {
		http.Error(w, "unexpected operation: "+op.Op, http.StatusBadRequest)
	}
	processRequest(w, r, op.Op, extraParam, true)
}
