package server

import (
	"asapo_broker/database"
	"asapo_common/utils"
	"encoding/json"
	"io/ioutil"
	"net/http"
	"strconv"
)

func routeWriteMessage(w http.ResponseWriter, r *http.Request) {
	body, err := ioutil.ReadAll(r.Body)
	if err != nil {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	keys := r.URL.Query()
	datasetSize := 0
	val, ok := keys["dataset_size"]
	if ok {
		datasetSize, err = strconv.Atoi(val[0])
		if err != nil {
			w.WriteHeader(http.StatusBadRequest)
			return
		}
	}

	id, err := strconv.Atoi(keys.Get("id"))
	if err != nil {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	var message database.MessageRaw
	if err := json.Unmarshal(body, &message); err != nil {
		http.Error(w, "failed to deserialize request body: "+err.Error(), utils.StatusWrongInput)
		return
	}
	message.Id = int64(id)

	extraParam := database.ExtraParamMessage{
		Id:          int64(id),
		DatasetSize: datasetSize,
		Message:     message}
	processRequest(w, r, "write_message", extraParam, false)
}

func routeWriteMeta(w http.ResponseWriter, r *http.Request) {
	body, err := ioutil.ReadAll(r.Body)
	if err != nil {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	keys := r.URL.Query()
	upsert, ok := getBoolKey(keys, "mode_upsert", true)
	if !ok {
		w.WriteHeader(http.StatusBadRequest)
		return
	}

	var message map[string]interface{}
	if err := json.Unmarshal(body, &message); err != nil {
		http.Error(w, "failed to deserialize request body: "+err.Error(), utils.StatusWrongInput)
		return
	}

	extraParam := database.ExtraParamMeta{
		Mode:    keys.Get("mode_op"),
		Id:      keys.Get("document_id"),
		Upsert:  upsert,
		Message: message}
	processRequest(w, r, "write_meta", extraParam, false)
}
