#!/bin/bash

# Usage:
#   bash run_asapo.sh [source_build_dir]
#
# Arguments:
#   source_build_dir  directory that contains the binaries to deploy.
#                     The default value is $project_project_root_dir/build where
#                     $project_project_root_dir is the root directory of the ASAPO project.

# Calculate the path of `project_root_dir` from the current script directory.
project_root_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && git rev-parse --show-toplevel)"
# The value of `source_build_dir` is either "$1" or "$project_root_dir/build"
# if no argument is provided to the script.
source_build_dir="${1:-$project_root_dir/build}"

cp "$source_build_dir"/authorizer/asapo-authorizer /asapo-authorizer
cp "$source_build_dir"/discovery/asapo-discovery /asapo-discovery
cp "$source_build_dir"/broker/asapo-broker /asapo-broker
cp "$source_build_dir"/file_transfer/asapo-file-transfer /asapo-file-transfer
cp "$source_build_dir"/receiver/receiver /receiver
cp "$source_build_dir"/monitoring/monitoring_server/asapo-monitoring-server /asapo-monitoring-server

cp "$project_root_dir"/deploy/asapo_services_light/*json /
cp "$project_root_dir"/deploy/asapo_services_light/envoy.yaml /

# Several receiver configs (tcp and fabric) are required for tests
cp "$project_root_dir"/deploy/asapo_services_light/receiver.json /receiver_tcp.json
cp "$project_root_dir"/deploy/build_env/services-linux/receiver_fabric.json /
cp "$project_root_dir"/deploy/build_env/services-linux/receiver_kafka.json /

cp "$project_root_dir"/deploy/build_env/services-linux/discovery.json /
cp "$project_root_dir"/deploy/build_env/services-linux/*key /
cp "$project_root_dir"/deploy/build_env/services-linux/supervisord.conf /etc/

cp "$source_build_dir"/deploy/asapo_services_light/nginx.conf /etc/nginx/nginx.conf

service slapd start
ldapadd -x -D "ou=rgy,o=desy,c=de" -f /etc/ldap/record.ldif -h localhost

cd / && supervisord --configuration /etc/supervisord.conf &
cd -

success=1
excluded_states=(STOPPED STARTING BACKOFF STOPPING EXITED FATAL UNKNOWN)
for i in {1..20}; do
    sleep 1
    supervisorctl status | grep -e $(sed 's/ / -e /g' <<<${excluded_states[@]})
    if [ $? -ne 0 ]; then
        success=0
        break
    fi
done
supervisorctl status
exit $success
