---
title: Data in ASAPO
---

### Data structure

All data that is produced, stored and consumed via ASAPO is structured on several levels.

#### Beamtime
This is the top level. Contains all data collected/produced during a single beamtime (Beamtime is the term used at DESY. Can also be Run, Experiment, Proposal, ...). Each beamtime has its own unique ID.

#### Data Source
During a beamtime, data can be produced by different sources. For example, a detector is a data source, if multiple detectors are used during an experiment, they can be different data sources or the same data source (more details below in datasets section). A user application that simulates or analyses data can also act as an ASAPO data source. Each data source has its own unique name within a beamtime.

#### Data Stream
Each data source can emit multiple data streams. For each specific data source name, stream names must be unique.

#### Message
Data streams consist of smaller entities - messages. The content of a message is quite flexible, to be able to cover a broad amount of use cases. Usually it is metadata and some binary data (e.g. a detector image, or an hdf5 file with multiple images). At the moment ASAPO itself is agnostic to the data and sees it as a binary array. Later some specific cases might be handled as well (the most prominent use case - an hdf5 file with multiple images).

An important aspect is that each message within a data stream must be assigned a consecutive integer index. Therefore, a streams always contain messages with index = 1,2,3 ... . This is different to traditional messaging systems where messages have timestamps or arbitrary unique hash IDs. The reason is that with timestamps the order of messages saved in the system might differ from the order the were generated by the data source (e.g. detector). And keeping correct order is required in many cases. Second reason is that it makes a random access to a specific message quite straightforward.

#### Datasets/Dataset substreams
In some cases multiple detectors are being used during an experiment. E.g. a 3D image is composed from multiple 2D images created by different detectors. In this case these 2D images can be composed to a dataset so that it a be processed later as a whole. One would then use a single data source (which would mean a set of detectors or "multi-detector" in this case), single data stream and, to compose a dataset, for each of it's components (each 2D image in our example) the corresponding detector would send a message with same id but to a different dataset substream.

So, for the case without datasets (single detector) the data hierarchy is Beamtime→Data Source → Data Stream → Message:

![Docusaurus](/img/data-in-asapo-workflow.png)

And with datasets (multi-detector) the data hierarchy is Beamtime→Data Source → Data Stream → Dataset→ Message in Dataset Substream:

![Docusaurus](/img/data-in-asapo-workflow2.png)

### Data storage

Data transferred to asapo service is stored in files on cache buffer or/and (depending on ingest mode) in disc. Asapo disc storage is bounded to DESY infrastructure. Mounting point is either located at beamline gpfs (so-called `raw` data type) or at core gpfs (so-called `processed` data type). Exact path is defined inside the asapo authorization system.

Currently, asapo caches data in a circular buffer with a size of multiple hundreds of GB. This cache is not dedicated to a certain producer, but rather global across asapo service.
