import asapo_producer


# callback snippet_start
def callback(payload, err):
    if err is not None and not isinstance(err, asapo_producer.AsapoServerWarning):
        # the data was not sent. Something is terribly wrong.
        print("could not send: ", payload, err)
    elif err is not None:
        # The data was sent, but there was some unexpected problem, e.g. the file was overwritten.
        print("sent with warning: ", payload, err)
    else:
        # all fine
        print("successfuly sent: ", payload)


# callback snippet_end

# create snippet_start
token = str(
    "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.e"
    "yJleHAiOjk1NzE3MTAyMTYsImp0aSI6Ind0ZmlzdGhpcyIsInN"
    "1YiI6ImJ0X2FzYXBvX3Rlc3QiLCJFeHRyYUNsYWltcyI6eyJBY"
    "2Nlc3NUeXBlcyI6WyJ3cml0ZSIsIndyaXRlcmF3IiwicmVhZCJ"
    "dfX0.cz6R_kVf4yh7IJD6bJjDdgTaxPN3txudZx9DE6WaTtk"
)

producer = asapo_producer.create_producer(
    endpoint="localhost:8400",
    type="processed",  # should be 'processed' or 'raw', 'processed' writes to the core FS
    beamtime_id="asapo_test",  # the folder should exist
    beamline="auto",  # can be 'auto', if beamtime_id is given
    data_source="test_source",
    token=token,
    nthreads=1,  # number of threads. Increase, if the sending speed seems slow
    timeout_ms=60000,
    instance_id="test_producer_instance",  # producer instance id (can be 'auto')
    pipeline_step="pipeline_step_1",
)

producer.set_log_level("error")  # other values are "warning", "info" or "debug".
# create snippet_end

# send snippet_start
# we are sending a message with with index 1 to the default stream. Filename must start with processed/
producer.send(
    id=1,  # message number. Should be unique and ordered.
    exposed_path="processed/test_file",  # name of the file. Should be unique, or it will be overwritten
    data=b"hello",  # binary data
    callback=callback,  # this is called after the asynchronous send operation finishes
)
# send snippet_end
# send data in loop

# add the following at the end of the script

# finish snippet_start
producer.wait_requests_finished(
    timeout_ms=2000
)  # will synchronously wait for all the data to be sent.
# Use it when no more data is expected.

# you may want to mark the stream as finished
producer.send_stream_finished_flag(
    stream="default",  # name of the stream. If you didn't specify the stream in 'send', it would be 'default'
    last_id=1,
)  # the number of the last message in the stream
# finish snippet_end
