#include "request_handler_db.h"
#include "../receiver_config.h"
#include "../receiver_logger.h"
#include "../request.h"
#include "asapo/common/internal/version.h"

namespace asapo {

RequestHandlerDb::RequestHandlerDb() : log__{GetDefaultReceiverLogger()} {
    // ToDo set endpoint in config
    asapo::SourceCredentials trusted_source{asapo::SourceType::kRaw, "instance", "step", "beamtime_id", "beamline", "subname", "token"};
    service_request_ = std::unique_ptr<ServiceRequest> {new ServiceRequest("localhost:8400", trusted_source)};
}

RequestInfo RequestHandlerDb::CreateBrokerApiRequest(const Request* request, std::string suffix) const{
        auto stream = request->GetStream().empty() ? "default" : request->GetStream();
        auto uri = "/" + kConsumerProtocol.GetBrokerVersion() + "/beamtime/" + request->GetBeamtimeId()
         + "/" + service_request_->UrlEscape(request->GetDataSource())
         + "/" + service_request_->UrlEscape(stream);
        if (suffix.size() > 0)
        {
            uri = uri + "/" + suffix;
        }

        RequestInfo ri;
        ri.api = uri;
        ri.extra_params += "&instanceid=" + service_request_->UrlEscape(request->GetProducerInstanceId());
        ri.extra_params += "&pipelinestep=" + service_request_->UrlEscape(request->GetPipelineStepId());
        ri.extra_params += "&id=" +  std::to_string(request->GetDataID());

        return ri;
}

StatisticEntity RequestHandlerDb::GetStatisticEntity() const {
    return StatisticEntity::kDatabase;
}

Error RequestHandlerDb::DBErrorToReceiverError(Error err) const {
    // ToDo implement conversion from broker error
    if (err == nullptr) {
        return nullptr;
    }
    Error return_err;
    return_err = ReceiverErrorTemplates::kInternalServerError.Generate("error from database");
    return_err->SetCause(std::move(err));
    return return_err;
}

}
