#include "request_handler_db_check_request.h"

#include "asapo/http_request/consumer_error.h"
#include "asapo/database/database.h"
#include "asapo/database/db_error.h"
#include "asapo/logger/logger.h"
#include "request_handler_db.h"
#include "../receiver_logger.h"
#include "../request.h"

namespace asapo {

RequestHandlerDbCheckRequest::RequestHandlerDbCheckRequest() {

}

Error RequestHandlerDbCheckRequest::GetRecordFromDb(const Request *request, MessageMeta *record) const {
    auto op_code = request->GetOpCode();
    auto id = request->GetDataID();

    RequestInfo ri =  RequestHandlerDb::CreateBrokerApiRequest(request, "0/"+std::to_string(id));
    ri.post = false;

    if (op_code == Opcode::kOpcodeTransferDatasetData) {
        auto id_in_set = request->GetCustomData()[1];
        ri.extra_params += "&dataset=true";
        ri.extra_params += "&minsize=1&substream=" + std::to_string(id_in_set);
    }
    Error err;
    auto response = service_request_->BrokerRequestWithTimeout(ri, &err, 0);

    if (!err && !record->SetFromJson(response)) {
        return ReceiverErrorTemplates::kProcessingError.Generate(std::string("malformed response:") + response);
    }

    return err;
}

bool RequestHandlerDbCheckRequest::SameRequestInRecord(const Request *request, const MessageMeta &record) const {
    std::string meta = request->GetMetaData();
    if (meta.size() == 0) {
        meta = "{}";  // so empty meta is stored in database
    }
    return request->GetDataSize() == record.size
        && request->GetFileName() == record.name
        && meta == record.metadata;
}

Error RequestHandlerDbCheckRequest::ProcessRequest(Request *request) const {

    MessageMeta record;
    auto err = GetRecordFromDb(request, &record);
    if (err) {
        return DBErrorToReceiverError(err == ConsumerErrorTemplates::kNoData ? nullptr : std::move(err));
    }

    if (SameRequestInRecord(request, record)) {
        return ReceiverErrorTemplates::kWarningDuplicatedRequest.Generate();
    } else {
        return ReceiverErrorTemplates::kBadRequest.Generate("already have record with same id");
    }
}

}
