#include "request_handler_db_fallback.h"
#include "../request.h"
#include "../receiver_logger.h"
#include "asapo/io/io_factory.h"
#include "asapo/common/internal/version.h"
#include "asapo/json_parser/json_parser.h"

namespace asapo {

StreamInfo GetStreamInfo(std::string record, Error* err) {

    std::vector<std::string> streams_encoded;
    Error parse_err;
    auto parser = JsonStringParser(std::move(record));
    *err = parser.GetArrayRawStrings("streams", &streams_encoded);
    if (*err || streams_encoded.size()==0 ) {
        return StreamInfo{};
    }
    if (streams_encoded.size()>1 ){
        *err =  GeneralErrorTemplates::kSimpleError.Generate("Server response contain more than one stream");
        return StreamInfo{};
    }

    StreamInfo si;
    auto ok = si.SetFromJson(streams_encoded[0]);
    if (!ok) {
        *err = GeneralErrorTemplates::kSimpleError.Generate("cannot parse " + streams_encoded[0]);
        return StreamInfo{};
    }
    return si;
}


Error RequestHandlerDbFallback::ProcessRequest(Request* request) const {

    auto stream = request->GetStream();
    auto op_code = request->GetOpCode();
    Error err;
    log__->Debug(RequestLog("Fallback request to broker", request));
    if (op_code == Opcode::kOpcodeStreamInfo || op_code == Opcode::kOpcodeLastStream) {
        RequestInfo ri = RequestHandlerDb::CreateBrokerApiRequest(request, "streams");
        ri.extra_params = "&detailed=true&from=" + service_request_->UrlEscape(stream);

        if (op_code == Opcode::kOpcodeStreamInfo) {
            ri.extra_params += "&single=true";
        } else {
            ri.extra_params += "&last=true";
        }

        auto response = service_request_->BrokerRequestWithTimeout(ri, &err, 1000);
        if (!err) {
            log__->Debug(RequestLog("get stream info from broker", request));
            auto stream_info = GetStreamInfo(response, &err);
            request->SetResponseMessage(stream_info.Json(), ResponseMessageType::kInfo);
        }
    }

    if (op_code == Opcode::kOpcodeDeleteStream){
        RequestInfo ri = RequestHandlerDb::CreateBrokerApiRequest(request, "delete");
        ri.post = true;
        DeleteStreamOptions options{};
        uint64_t flag = request->GetCustomData()[0];
        options.Decode(flag);
        ri.body = options.Json();
        service_request_->BrokerRequestWithTimeout(ri, &err, 1000);
    }

    if (op_code == Opcode::kOpcodeGetMeta){
        auto stream_name = request->GetStream();
        std::string meta_type = stream_name.empty() ? "0" : "1";
        RequestInfo ri = RequestHandlerDb::CreateBrokerApiRequest(request, "0/meta/"+meta_type);
        auto result = service_request_->BrokerRequestWithTimeout(ri, &err, 1000);
        if (!err) {
            request->SetResponseMessage(result, ResponseMessageType::kInfo);
        }
    }

    if (op_code == Opcode::kOpcodePersistStream){
        RequestInfo ri = RequestHandlerDb::CreateBrokerApiRequest(request, "persist");
        ri.post = true;
        service_request_->BrokerRequestWithTimeout(ri, &err, 1000);
    }
    return DBErrorToReceiverError(std::move(err));
}

}
