#include "request_handler_db_meta_write.h"
#include "../request.h"
#include "../receiver_logger.h"
#include "asapo/io/io_factory.h"
#include "asapo/common/internal/version.h"

namespace asapo {

Error RequestHandlerDbMetaWrite::ProcessRequest(Request* request) const {
    auto size = request->GetDataSize();
    auto meta = request->GetData();
    auto api_version = VersionToNumber(request->GetApiVersion());

    std::string stream;
    MetaIngestMode mode;
    if (api_version < 3) {
        // old approach, deprecates 01.07.2022
        mode.op = MetaIngestOp::kReplace;
        mode.upsert = true;
    } else {
        stream = request->GetStream();
        mode.Decode(request->GetCustomData()[kPosMetaIngestMode]);
    }

    log__->Debug(RequestLog("Send meta to broker", request));

    RequestInfo ri =  RequestHandlerDb::CreateBrokerApiRequest(request, "write_meta");
    ri.post = true;
    ri.body = std::string(static_cast<const char*>(meta), size);
    ri.extra_params += "&mode_op=" + std::to_string(static_cast<int>(mode.op));
    std::string upsert = mode.upsert ? "true" : "false";
    ri.extra_params += "&mode_upsert=" + upsert;
    auto id = stream.empty() ? "bt" : "st_" + stream;
    ri.extra_params += "&document_id=" + id;
    Error err;
    service_request_->BrokerRequestWithTimeout(ri, &err, 1000);
    return DBErrorToReceiverError(std::move(err));
}
RequestHandlerDbMetaWrite::RequestHandlerDbMetaWrite() {

}

}
