#include <gtest/gtest.h>
#include <gmock/gmock.h>

#include "asapo/unittests/MockIO.h"
#include "asapo/unittests/MockDatabase.h"
#include "asapo/unittests/MockLogger.h"
#include "asapo/unittests/MockHttpClient.h"

#include "../../src/receiver_error.h"
#include "../../src/request.h"
#include "../../src/request_handler/request_factory.h"
#include "../../src/request_handler/request_handler.h"
#include "../../src/request_handler/request_handler_db.h"
#include "asapo/common/networking.h"
#include "../mock_receiver_config.h"
#include "asapo/common/data_structs.h"

#include "../receiver_mocking.h"

using namespace testing;
using namespace asapo;


namespace {

class DbHandlerTests : public Test {
  public:
    std::string expected_collection_name = "test";
    std::unique_ptr<NiceMock<MockRequest>> mock_request;
    NiceMock<asapo::MockLogger> mock_logger;
    ReceiverConfig config;
    std::string expected_beamtime_id = "beamtime_id";
    std::string expected_stream = "source";
    std::string expected_default_source = "detector";
    std::string expected_discovery_server = "discovery";
    std::string expected_database_server = "127.0.0.1:27017";

    MockHttpClient mock_http_client;

    void SetUp() override {
        GenericRequestHeader request_header;
        request_header.data_id = 2;
        mock_request.reset(new NiceMock<MockRequest> {request_header, 1, "", nullptr, nullptr});
        ON_CALL(*mock_request, GetBeamtimeId()).WillByDefault(ReturnRef(expected_beamtime_id));
        ON_CALL(*mock_request, GetDataSource()).WillByDefault(ReturnRef(expected_stream));

    }
    void MockAuthRequest(bool error, HttpCode code = HttpCode::OK);

};


void DbHandlerTests::MockAuthRequest(bool error, HttpCode code) {
    if (error) {
        EXPECT_CALL(mock_http_client, Get_t(expected_discovery_server + "/asapo-mongodb",  _, _)).
        WillOnce(
            DoAll(SetArgPointee<2>(asapo::GeneralErrorTemplates::kSimpleError.Generate("http error").release()),
                  Return("")
                 ));
        EXPECT_CALL(mock_logger, Error(AllOf(HasSubstr("discovering database server"),
                                             HasSubstr("http error"),
                                             HasSubstr(expected_discovery_server))));

    } else {
        EXPECT_CALL(mock_http_client, Get_t(expected_discovery_server + "/asapo-mongodb", _, _)).
        WillOnce(
            DoAll(
                SetArgPointee<1>(code),
                SetArgPointee<2>(nullptr),
                Return(expected_database_server)
            ));
        if (code != HttpCode::OK) {
            EXPECT_CALL(mock_logger, Error(AllOf(HasSubstr("failure discover database server"),
                                                 HasSubstr("http code"),
                                                 HasSubstr(std::to_string(int(code))),
                                                 HasSubstr(expected_discovery_server))));
        } else {
            EXPECT_CALL(mock_logger, Debug(AllOf(HasSubstr("discovered"),
                                                 HasSubstr(expected_database_server))));
        }
    }


}

}
