#!/usr/bin/env bash

# Function to download files from GitHub. Requires `curl`.
# Usage:
#   github_download_files <target_dir> <repo_url> <commit_sha1> [file1 ...]
# Example:
#   source scripts/github.sh
#   github_download "3d_party/fastcov" "https://github.com/RPGillespie6/fastcov" \
#     "40dffe81d62c0d897afe4108f3b5489487ff3bce" "fastcov.py" "LICENSE" "README.md"
function github_download_files() {
  local target_dir=$1
  local repo_url=$2
  local commit_sha1=$3
  shift 3
  echo "Installing from commit $repo_url/commit/$commit_sha1 into $target_dir"
  # Raw files are stored in `raw.githubusercontent.com` instead of `github.com`
  local raw_url="${repo_url//github.com/raw.githubusercontent.com}"
  local error_code=0
  for filepath in "$@"; do
    local src="$raw_url/$commit_sha1/$filepath"
    local dst="$target_dir/$filepath"
    # If the file is inside a subdirectory, the script needs to create the subdirectory
    # on the local filesystem before calling CURL.
    local dst_dir
    dst_dir="$(dirname "$dst")"
    mkdir -p "$dst_dir"
    printf "  - %s\t--> %s\n" "$src" "$filepath"
    curl --location --fail --show-error --silent --output "$dst" "$src" || error_code=$?
  done
  return $error_code
}
