#!/usr/bin/env bash

set -e

trap Cleanup EXIT

producer_bin=$1
consumer_bin=$2
asapo_tool_bin=$3
network_type=$4

beamtime_id=asapo_test

monitor_database_name=db_test
proxy_address=127.0.0.1:8400

beamline=test
receiver_root_folder=/tmp/asapo/receiver/files
facility=test_facility
year=2019
receiver_folder=${receiver_root_folder}/${facility}/gpfs/${beamline}/${year}/data/${beamtime_id}


Cleanup() {
    echo cleanup
    rm -rf ${receiver_root_folder}
    rm -rf out
    echo "db.dropDatabase()" | mongo ${beamtime_id}_detector

    set +e
    influx_out=`influx -database ${monitor_database_name} -execute "drop series from statistics, RequestsRate"`
    influx_status=$?
    echo "Influx output: ${influx_out}"
    if [ $influx_status -ne 0 ]; then
        if [[ $influx_out == *"401"* ]]; then
            echo "Ignoring auth error from influxdb"
        else
            echo "influxdb failed to delete test database '${monitor_database_name}'"
            exit $influx_status
        fi
    fi
}

token=`$asapo_tool_bin token -endpoint http://localhost:8400/asapo-authorizer -secret admin_token.key -types read $beamtime_id`

echo "Start producer"
mkdir -p ${receiver_folder}
$producer_bin localhost:8400 ${beamtime_id} 100 100 4 0 100 5 &

echo "Start consumer in metadata only mode"
$consumer_bin ${proxy_address} ${receiver_folder} ${beamtime_id} 2 $token 5000 1 1 | tee out
grep "Processed 100 dataset(s)" out
grep "with 500 file(s)" out
