---
title: Comparison to Other Solutions
---

Here we consider how ASAPO is different from other workflows practiced at DESY. The possible candidates are:

### Filesystem
Probably the most often used approach for now. Files are written to the beamline filesystem directly via NFS/SMB mount or by HiDRA and copied to the core filesystem by a copy daemon. A user (software) then reads the files from the filesystem.

### Filesystem + Kafka
Previous workflow + there is a Kafka instance that produces messages when a file appears in the core filesystem. These messages can then be consumed by user software.

### HiDRA
HiDRA can work in two modes - wether data is transferred via it or data is written over NFS/SMB mounts and HiDRA monitors a folder in a beamline filesystem. In both case one can subscribe to HiDRA's data queue to be informed about a new file.

### ASAPO

ASAPO does not work with files, rather with data streams. Well, behind the scenes it does use files, but in principle what a user see is a stream of messages, where a message typically consists of metadata and data blobs. Data blob can be a single image, a file with arbitrary content or whatever else, even null. Important is - what goes to one end, appears on the other. And that each message must have a consecutive index. These messages must be ingested to an ASAPO data stream in some way (e.g. using ASAPO Producer API or HiDRA) and data, if not yet, will be transferred and stored in the data center. A user can then read the messages from the stream and process it in a way he likes.

### Compare by categories

In the table below we compare the approaches from different points of view and in the [next table](#compare-by-features) we compare the approaches by available features.

|     Category     |                                                                            Filesystem                                                                           |                                                                                                                                                                                                                            Filesystem+Kafka                                                                                                                                                                                                                           |                                                                                                                                              HiDRA                                                                                                                                             |                                                                                                                                                                                                                                                                                                                                                            ASAPO                                                                                                                                                                                                                                                                                                                                                           |
|----------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Data ingest      | Traditional way - write to disk                                                                                                                                 | Same as Filesystem, additionally a message is send to a Kafka topic with basic file metadata (name, size, creation date, ...)                                                                                                                                                                                                                                                                                                                                         | Outperforms data ingest via NFS/SMB, uses ZeroMQ(TCP), saturates 10GE bandwidth. To be tested with 100GE. Same metadata as with Kafka                                                                                                                                                          |Uses parallel TCP connections & in the future RDMA. Can add arbitrary metadata (JSON format). Can send data to various streams to create arbitrary data hierarchy (e.g. stream per scan). Saves data to a memory buffer and on disk.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| Offline analysis | Traditional way - read from disk. Need to know the filename to read, usually not a problem.                                                                     | Same as Filesystem, there is no benefits reading Kafka queue after all data is arrived                                                                                                                                                                                                                                                                                                                                                                                | Not possible, need to fallback to Filesystem                                                                                                                                                                                                                                                   | Same as online analysis (see below)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| Online analysis  | No efficient way to recognise that new data is available - periodically read folder content and compare with previous state?, periodically check file appeared? | Using a subscription to a "new files" topic, a user software can be made aware of new data quite soon and react correspondingly.                                                                                                                                                                                                                                                                                                                                      | One can subscribe to a HiDRA's ZeroMQ stream and consume data. If data arrives faster than it can be processed or e.g. user software crashes - data might be skipped.                                                                                                                          | One can get data from various streams in different ways - get next unprocessed message ordered by index, get last, get by id. Since everything is stored in persistent storage, processing is possible with arbitrary slow (but also fast) consumers. Resilient to connections loss or consumer crashes.                                                                                                                                                                                                                                                                                                                                                                                                                   |
| Performance      | As good as read/write to disk. Can be an issue, especially with new detectors 100GE+ networks.                                                                  | As good as read/write to disk + some latency for the file to be written to the beamline filesystem, copied to the core filesystem and a message to go through Kafka.                                                                                                                                                                                                                                                                                                  | Data is available as soon as it is received by HiDRA. If multiple consumer groups want to read same data (one consumer is known - the process that writes the file), data will be transferred multiple times, which influences the network performance.                                        | Data is available to be consumed as soon as it has arrived and saved to beamline filesystem (later can be optimised by using persistent memory instead). No need to read from disk since it also remains in memory buffer.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| Parallelisation  | Parallelisation is easily possible e.g. with an MPI library.                                                                                                    | Parallelisation is possible if Kafka's topics are partitioned (which is not the case in the moment)                                                                                                                                                                                                                                                                                                                                                                   | Not out of the box, possible with some changes from user's side                                                                                                                                                                                                                                | Parallelisation is easily possible, one can consume data concurrently with different consumers from the same stream. Normally, synchronisation between consumers is not needed, but this might depend on a use case. When configured, data can be resent if not acknowledged during a specified time  period.                                                                                                                                                                                                                                                                                                                                                                                                              |
| Search/filter    | Hardly possible, manually parsing some metadata file, using POSIX commands?                                                                                     | Same as Filesystem. There is Kafka SQL query language which could be used if there would be metadata in messages, which is not the case (yet?).                                                                                                                                                                                                                                                                                                                       | Not possible                                                                                                                                                                                                                                                                                   | One can use a set of  SQL queries to ingested metadata.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| General comments | Might be ok for slow detectors or/and a use case without online analysis requirements. Might be the only way to work with legacy applications                   | Fits well for the cases where a software just needs a trigger that some new data has arrived, when processing order, extra metadata, parallelisation is not that important or implemented by other means. Some delay between an image is generated and the event is emitted by Kafka is there, but probably not that significant (maximum a couple of seconds). Might be not that appropriate for very fast detectors since it is still using filesystem to write/read data. | Works quite well to transfer files from detector to the data center. Also a good candidate for live viewers, where the last available "image" should be displayed. Does not work for offline analysis or for near real-time analysis where image processing can take longer than image taking. | Tries to be a general solution which improves in areas where other approaches not suffice: single code for offline/near real-time/online analysis, parallelisation, extended metadata, efficient memory/storage management, getting data without access to filesystem (e.g. from detector pc without core filesystem mounted), computational pipelines, ... Everything has its own price: user software must be modified to use ASAPO, a wrapper might be needed for legacy software that cannot be modified, user/beamtime scientist should better structure the data - e.g. consecutive indexes must be available for each image, one has to define to which stream to write/read data, what is the format of the data, ... |


### Compare by features

| Feature                                                                                                            | Filesystem                         | Filesystem+Kafka                   | HiDRA                              | ASAPO                              |
|--------------------------------------------------------------------------------------------------------------------|------------------------------------|------------------------------------|------------------------------------|------------------------------------|
| Send metadata with image                                                                                           | No                                 | No                                 | No                                 | Yes                                |
| Get last image                                                                                                     | No                                 | No                                 | Yes                                | Yes                                |
| Get image by id                                                                                                    | No                                 | No                                 | No                                 | Yes                                |
| Get image in order                                                                                                 | No                                 | No                                 | No                                 | Yes                                |
| Immediately get informed that a new image is arrived                                                               | No                                 | Yes                                | Yes                                | Yes                                |
| Access image remotely, without reading filesystem                                                                  | No                                 | No                                 | Yes, if it is still in buffer      | Yes                                |
| Access past images                                                                                                 | Yes                                | Yes                                | No                                 | Yes                                |
| Need to change user code                                                                                           | No                                 | Yes                                | Yes                                | Yes                                |
| Parallelisation                                                                                                    | Yes (if user software allows that) | Not out of the box                 | Not out of the box                 | Yes                                |
| Legacy applications                                                                                                | Yes                                | No (wrapper could be a workaround) | No (wrapper could be a workaround) | No (wrapper could be a workaround) |
| Transparent restart/continuation of simulations in case e.g. worker process crashes, also for parallel simulations | Not out of the box                 | Yes                                | No                                 | Yes                                |
